﻿/*
 * NOTE:    DO NOT POST ANY MODIFIED VERSIONS OF THIS TO THE FORUMS.
 * 
 *          DO NOT UTILIZE ANY PORTION OF THIS PLUGIN WITHOUT
 *          THE PRIOR PERMISSION OF AUTHOR.  PERMITTED USE MUST BE
 *          ACCOMPANIED BY CREDIT/ACKNOWLEDGEMENT TO ORIGINAL AUTHOR.
 * 
 * SetTheLeader Plugin
 * 
 * Author:  Bobby53
 * 
 */

// #define HIDE_PLAYER_NAMES
// #define DEBUG

using System.Linq;
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Styx;
using Styx.Helpers;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.Plugins.PluginClass;
using System.Threading;
using Styx.Logic.Combat;
using Styx.Logic;

namespace NSetLead
{
    public class CSetLead : HBPlugin
    {

        private static readonly LocalPlayer Me = ObjectManager.Me;
        // public static SetTheLeader local = null;
        private static bool firstCallToPulse = true;
        private static WoWPlayer userSelectedLeader = null;
        private static BotEvents.OnBotStartDelegate btStrt;

        public static List<WoWPlayer> groupList
        {
            get { return Me.IsInRaid ? Me.RaidMembers : Me.PartyMembers; }
        }


        public void Initialize()
        {
            if (!firstCallToPulse)
                return;

            firstCallToPulse = false;
            Slog("Enabled");

            Lua.Events.AttachEvent("PARTY_MEMBERS_CHANGED", HandlePartyMembersChanged);
            Lua.Events.AttachEvent("READY_CHECK", HandleReadyCheck);
            Lua.Events.AttachEvent("RAID_INSTANCE_WELCOME", HandleRaidInstanceWelcome);
            Lua.Events.AttachEvent("RAID_ROSTER_UPDATE", HandleRaidRosterUpdate);

            btStrt = new BotEvents.OnBotStartDelegate(startBot);
            BotEvents.OnBotStart += btStrt;

            if (DoWeNeedToScan())
            {
                Slog("{0} had someone joined/left -- rescanning", Me.IsInRaid ? "Raid" : "Party");
                ScanForBestLeader();
            }
        }

        public override void Pulse()
        {
            if (firstCallToPulse)
            {
                firstCallToPulse = false;
                Initialize();
            }
        }


        public static void Slog(string format, params object[] args)
        {
            Logging.Write("[SetTheLeader]: " + format, args);
        }

        public static void Dlog(string format, params object[] args)
        {
#if DEBUG
            Logging.WriteDebug("[SetTheLeader]: " + format, args);
#endif
        }

        public static string Safe_UnitName(WoWUnit unit)
        {
            if (unit == null)
                return "(null)";

#if HIDE_PLAYER_NAMES
            return unit.Class.ToString() + " (max health:" + unit.MaxHealth + ")";
#else
            return unit.Name + " (max health: " + unit.MaxHealth + ")";
#endif
        }


        //private delegate void startBot();
        private void startBot(EventArgs args)
        {
            if (DoWeNeedToScan())
            {
                Slog("Bot restarted -- rescanning");
                ScanForBestLeader();
            }
        }

        private static void HandlePartyMembersChanged(object sender, LuaEventArgs args)
        {
            if (DoWeNeedToScan())
            {
                Slog("{0} had someone joined/left -- rescanning", Me.IsInRaid ? "Raid" : "Party");
                ScanForBestLeader();
            }
        }

        private static void HandleReadyCheck(object sender, LuaEventArgs args)
        {
            if (DoWeNeedToScan())
            {
                Slog("Ready Check -- rescanning");
                ScanForBestLeader();
            }
        }

        private static void HandleRaidInstanceWelcome(object sender, LuaEventArgs args)
        {
            if (DoWeNeedToScan())
            {
                Slog("Entered Instance -- rescanning");
                ScanForBestLeader();
            }
        }

        private static void HandleRaidRosterUpdate(object sender, LuaEventArgs args)
        {
            if (DoWeNeedToScan())
            {
                Slog("Group Roster Update -- rescanning");
                ScanForBestLeader();
            }
        }


        public override void OnButtonPress()
        {
            var configFrm = new ConfigForm();

            Initialize();  // call in case options pressed before we get Pulse()

            if ( DialogResult.OK == configFrm.ShowDialog())
            {
                if ( Me.IsInParty && Me.IsGroupLeader)
                {
                    Slog("Set Group Leader to {0}", Safe_UnitName(configFrm.chosenPlayer));
                    PromoteToLeader(configFrm.chosenPlayer);
                }

                Slog("Set RAF Leader to {0}", Safe_UnitName(configFrm.chosenPlayer));
                RaFHelper.SetLeader(configFrm.chosenPlayer);
            }
        }


        private static bool DoWeNeedToScan()
        {
            if (RaFHelper.Leader == null || !groupList.Contains(RaFHelper.Leader))
                return true;

            return RaFHelper.Leader != userSelectedLeader && "TANK" != GetGroupRoleAssigned(RaFHelper.Leader);
        }

        private static void ScanForBestLeader()
        {
            WoWPlayer leader = FindFirstPlayerWithTankRole();
            if (leader != null)
                Slog("Chose player with tank role {0}", Safe_UnitName(leader));
            else
            {
                leader = FindPlayerWithMaxHealth();
                if (leader != null)
                    Slog("Chose player with greatest health {0}", Safe_UnitName(leader));
                else
                    Slog("No suitable players found, clearing leader");
            }

            if (leader == null)
                RaFHelper.ClearLeader();
            else
                RaFHelper.SetLeader(leader);

            return;
        }

        private static WoWPlayer FindFirstPlayerWithTankRole()
        {
            return (from p in groupList where !p.IsMe && "TANK" == GetGroupRoleAssigned(p) select p).FirstOrDefault();
        }

        public static string GetGroupRoleAssigned(WoWPlayer p)
        {
            if (!Me.IsInParty && !Me.IsInRaid)
                return "NONE";

            string luaCmd = string.Format("return UnitGroupRolesAssigned(\"" + p.Name + "\")");
            string role = Lua.GetReturnVal<string>(luaCmd, 0);
            Dlog("{0} : {1}", role, Safe_UnitName(p));
            return role;
        }

        public static bool IsPartyLeader(WoWPlayer p)
        {
            string luaCmd = string.Format("return IsPartyLeader()");
            bool isLeader = Lua.GetReturnVal<bool>(luaCmd, 0);
            Dlog("{0} : {1} the leader", Safe_UnitName(p), isLeader ? "is" : "is not");
            return isLeader;
        }

        public static void PromoteToLeader(WoWPlayer p)
        {
            string luaCmd = string.Format("PromoteToLeader(\"" + p.Name + "\")");
            Lua.DoString(luaCmd);
        }

        private static WoWPlayer FindPlayerWithMaxHealth()
        {
            return (from p in groupList where !p.IsMe orderby p.MaxHealth descending select p).FirstOrDefault();
        }


        public override string Name { get { return "Set The Leader"; } }
        public override string Author { get { return "Bobby53"; } }
        public override Version Version { get { return new Version(0,9,1); } }
        public override string ButtonText { get { return "SetTheLeader..."; } }
        public override bool WantButton { get { return true; } }

    }

}


